////////////////////////////////////////////////////////
// File:     alicebob.c 
// Purpose:  example lock code
// Compile:  gcc -lpthread -Wall -g -o alicebob alicebob.c
//
// Author:   Eric Kee, w/special thanks to: Wei Pan
// Class:    CS78, Computer Networks, Spring 2008
//
// Comments:
//   This program is intended to give a simple 
//   introduction to using pthreads threads and mutexes.
//   This program creates two threads, Alice and Bob, 
//   That use locks to control access to a shared 
//   resource.  In this simple case, the shared resource
//   is an integer.  
//
////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>        // Gives us the sleep function
#include <pthread.h>       // Including pthread API


// Globally shared variables
pthread_mutex_t car_lock = PTHREAD_MUTEX_INITIALIZER;   // Lock to control access the "car"
int car = 0;                                            // The supposed "car"



// The Alice function is going to write her driving speed into 
// the car variable and the print that variable's value to the 
// screen.  If she does this without acquiring the car_lock 
// first, though, there's a chance that bob will set the car
// variable equal to the number of tires.  If Bob changes the
// car variable just before Alice calls printf, alice will
// report the wrong driving speed!  
//
// None of these bad things happen, though, because we use locks
void* AliceFunc(void *ptr) 
{
  pthread_mutex_lock(&car_lock);                    // Get lock to get into the car
  car = 1000;                                       // use the shared resource "car"
  printf("Alice:  Ok, i'm goin' %i miles an hour...\n", car);
  sleep(1);                                         // Do stuff here
  printf("Alice:  Ok, i'm done with the car.\n");
  pthread_mutex_unlock(&car_lock);                  // Release lock to get into the car!

  return NULL; 
}//end AliceFunc()


// The Bob function is going to write the number of tires that
// bob is changing into the car variable and print that variable's
// value to the screen.  If Alice changes the car variable just
// before Bob, though, bob will report that he is changing 1000 
// tires!
// 
// This bad case won't ever happen, though...
void* BobFunc(void* ptr)
{
  pthread_mutex_lock(&car_lock);                    // Get lock to get into the car
  car = 4;                                          // use the shared resource "car"
  printf("Bob:  Ok, i'm changin' the %i tires... \n", car);
  sleep(1);                                         // Do stuff here
  printf("Bob:  Ok, i'm done with the tires.\n");
  pthread_mutex_unlock(&car_lock);                  // Release lock to get into the car!

  return NULL;
}//end BobFunc()


// Main will be our first thread
int main() 
{
  int rc;                                           // For return codes
  pthread_t AliceThread;                            // Two thread variables for
  pthread_t BobThread;                              //  ..Alice and Bob
   
  // Create alice and bob
  rc = pthread_create(&AliceThread, NULL, AliceFunc, NULL);
  rc = pthread_create(&BobThread, NULL, BobFunc, NULL);

  // Wait for Alice and Bob to complete
  rc = pthread_join(AliceThread, NULL);       
  rc = pthread_join(BobThread, NULL);

  exit(0);                                          // Never reached
}//end main
