////////////////////////////////////////////////////////
// File:     threads.c 
// Purpose:  example pthreads code
// Compile:  gcc -pthread -Wall -g -o threads threads.c
//
// Author:   Eric Kee, w/special thanks to: Wei Pan
// Class:    CS78, Computer Networks, Spring 2008
//
// Comments:
//   This program gives a hello-world introduction to 
//   using pthreads.  When this program runs, the user
//   will see output that is generated by printf 
//   statements from one thread while a second thread
//   updates the value being printed.  This demonstrates
//   the concept of a *shared address space* between
//   pthreads--a shared address space means that global
//   variables are shared between the threads.
//
//   This code also demonstrates how to pass variables 
//   into thread functions.  By passing a pointer 
//   into thread upon thread creation, a thread may 
//   modify any variable that is referenced by the 
//   pointer, whether the variable is global or local. 
//
////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>        // Gives us the sleep function
#include <pthread.h>       // Including pthread API

/////////////////////////////
int i;                     // Global variables are SHARED!
/////////////////////////////


// A concurrent function/////
void* print_i(void *ptr) 
{
  while (1)                // While forever...      
  {
    sleep(1);              // Sleep only 1 second
    printf("%d\n", i);     // And print the shared variable's value
  }
}//end print_i(...)//////////


// Main will be our first thread
int main() 
{
  pthread_t t1;
  int rc;                  // Used to catch return codes
  i = 1;
  
  // Create the thread!  In practice you must check
  // the return value to make sure there wasn't a creation
  // error.
  rc = pthread_create(&t1, NULL, print_i, NULL);

  while (1)                // While forever...
  {
    sleep(2);              // Wait 2 seconds
    i = i + 1;             // And then increment the shared variable
  }

  exit(0);                 // Never reached
}//end main
