////////////////////////////////////////////////////////
// File:     timer.c 
// Purpose:  example timer code
// Compile:  gcc -pthread -Wall -g -o timer timer.c
//
// Author:   Eric Kee
// Class:    CS78, Computer Networks, Spring 2008
//
// Comments:
//   This program demonstrates how to use a thread to
//   create a timer.  In this program our thread
//   function has a hard-coded time; however, the child
//   could sleep for a dynamic length of time. Simply
//   pass the number of seconds as a parameter.  To 
//   figure out how to do this, read the man pages for
//   pthread_create(...)
//
////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>        // Gives us the sleep function
#include <pthread.h>       // Including pthread API

int timer_done = 0;        // A variable to indicate when time is up


// A concurrent timer function
// You might notice that we don't use a lock to synchronize 
// access to the global timer_done variable.  A lock is not
// needed in this special case because parent process will
// never write to the lock and this function will never read
// from it.  No bad interleavings can occur.
void* TimerFunc(void *ptr) 
{
  printf("Child: Sleeping 5 seconds before setting timer flag.\n");
  sleep(5);                // Set timer to go off in 5 seconds
  printf("Child: Time's up!  Setting flag.\n");
  timer_done = 1;          // Signal that the time is up

  return NULL;
}//end print_i(...)//////////


// Main will be our first thread
int main() 
{
  pthread_t timer_thread;  // A thread for the timer function
  int rc;                  // Used to catch return codes

  timer_done = 0;          // Initialize the timer to off and start the thread
  rc = pthread_create(&timer_thread, NULL, TimerFunc, NULL);

  while (1)                // While forever...
  {
    // Do important stuff that takes some time
    printf("Parent: Sleeping 1 sec before polling the flag...\n");
    sleep(1);              // Sleep to simulate us doing important things
    if (timer_done == 1)
      break;
  }

  printf("Parent: Timer set flag. Exiting.\n");
  exit(0);                 // Never reached
}//end main
