//////////////////////////////////////////////////////////////////////
// File:     unpredictable.c 
// Purpose:  To demonstrate thread unpredictability
// Compile:  gcc -pthread -Wall -g -o unpredictable unpredictable.c
//
// Author:   Eric Kee, w/special thanks to: Sean Smith
// Class:    CS78, Computer Networks, Spring 2008
// Date:     2008.04.16
//
// Comments:
//   This program demonstrates how threads can be 
//   unpredictable.  The output generated by this code
//   varies randomly depending upon the execution order
//   of the statements. 
//
//   We use a special function, "consider_interleaving"
//   that randomly sleeps for 1 second or it doesn't sleep
//   at all.  While we use consider_interleaving to force random 
//   execution orders for this demo, ...
//     YOUR CODE WILL INTERLEAVE ITSELF RANDOMLY AT 
//     THE MOST UNEXPECTED TIMES! 
//        E.G. WHEN I AM *GRADING* YOUR WORK!  
//    
//     WARNING: Threads can cause Heisenbugs... 
//     http://en.wikipedia.org/wiki/Heisenbug#Heisenbugs
//
//////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>                     // Including pthread API

// This function is used to insert random sleeps so you don't have
// to run the code 1000s of times to find the cases that produce
// funny output.
void consider_interleaving(void);       
int  flip_coin(void);


// This function will run concurrently.
void* print_i(void *ptr) 
{
  consider_interleaving();
  printf("a\n");
  consider_interleaving();
  printf("b\n");
  consider_interleaving();

  return NULL;
}//end print_i(...)



int main() 
{
  pthread_t t1;
  int rc;                        // Used to catch return codes

  rc = pthread_create(&t1, NULL, print_i, NULL);

  consider_interleaving();
  printf("c\n");
  consider_interleaving();
  printf("d\n");
  consider_interleaving();

  return 0;
}//end main()




//--------------------------------------------------------
// Source: Sean Smith (thanks!)
// first call: initialize random_byte
// next 8 calls: return a random bit
//---------------------------------------------------
int flip_coin(void) 
{

  static int  fd;
  static int initialized = 0;
  static int bits_left = 0;
  static unsigned char random_byte;
  int random_bit;

  if (!initialized) {
    
    fd = open("/dev/random", O_RDONLY);
    if (fd < 0) {
      fprintf(stderr,"open failed\n");
      exit(-1);
    }

    initialized = 1;
  }

  if (0 == bits_left) {
    if (sizeof(random_byte) != read(fd, &random_byte, sizeof(random_byte))) {
      fprintf(stderr, "read failed\n");
      exit(-1);
    }
    bits_left = 8;
  }

  random_bit = random_byte & 0x01;
  random_byte >>= 1;
  bits_left--;

  return random_bit;
}


// Source: Sean Smith
// Call this function to enter a randomized sleep
void consider_interleaving(void) {

  if (flip_coin())
    sleep(1);
}
